#include "audiolib.h"

static int currentframe,remainframe;

Audio::Audio()
{
}

Audio::~Audio()
{
kill(artsd,SIGTERM);
}

bool Audio::Init()
{
mixer = open("/dev/mixer",O_RDWR);

int send[2], recv[2];

socketpair(AF_UNIX, SOCK_STREAM, 0, send);
socketpair(AF_UNIX, SOCK_STREAM, 0, recv);

//signal(SIGPIPE, SIG_IGN);
signal(SIGCHLD, SIG_IGN);

artsd = fork();

if( artsd == 0)
	{
	dup2(send[0], STDIN_FILENO);
	close(send[0]);
	close(send[1]);
	dup2(recv[0], STDOUT_FILENO);
	dup2(recv[0], STDERR_FILENO);
	close(recv[0]);
	close(recv[1]);
	execlp("artsd",NULL);
	}

close(send[0]);
arts_sender = send[1];
close(recv[0]);
arts_receiver = recv[1];
}

void Audio::Kill()
{
kill(artsd,SIGTERM);
}


int Audio::SetVolume(int offset)
{
int vol;

GetVolume();
right +=  offset;
left +=  offset;
vol = (right << 8 ) + (left);

ioctl(mixer, MIXER_WRITE(SOUND_MIXER_PCM), &vol);

return GetVolume();
}

int Audio::GetVolume()
{
int vol;
ioctl(mixer, MIXER_READ(SOUND_MIXER_PCM), &vol);
left  =  vol & 0xff;
right = (vol & 0xff00) >> 8;

vol  = (left + right)/2;

return vol;

}

Mp3::Mp3()
{
playing = false;
Quee.clear();
}

Mp3::~Mp3()
{
kill(mpg123,SIGTERM);
}

bool Mp3::Init()
{
int send[2], recv[2];

socketpair(AF_UNIX, SOCK_STREAM, 0, send);
socketpair(AF_UNIX, SOCK_STREAM, 0, recv);

signal(SIGPIPE, SIG_IGN);
signal(SIGCHLD, SIG_IGN);


mpg123 = fork();

if( mpg123 == 0)
	{
	dup2(send[0], STDIN_FILENO);
	close(send[0]);
	close(send[1]);
	dup2(recv[0], STDOUT_FILENO);
	dup2(recv[0], STDERR_FILENO);
	close(recv[0]);
	close(recv[1]);
	execlp("mpg123","329Player","-R","-",NULL);
	}

close(send[0]);
sender = send[1];
close(recv[0]);
receiver = recv[1];

usleep(500);
if (mpg123 == -1)
	 {
	if (sender) close(sender);
	if (receiver) close(receiver);
	sender = receiver = 0;
     	return false;
   	}
else
	{
	buflen = 0;
	setpriority(PRIO_PROCESS,mpg123,-20);
	return true;
	}
}

void Mp3::Open(string filename)
{
char s[512];
int offset = 0;

if(filename == "PREV")offset = -2;
else offset = 0;

if(lst > 0 && (ready + offset) <= lst && (ready+offset) >= 0)
	{
	sprintf(s, "LOAD %s\n",Quee[ready+offset].c_str());
	ready+= offset+1;
	}
else
if(lst == 0)
	{
	sprintf(s, "LOAD %s\n",filename.c_str());
	}
playing = true;

write(sender,s,strlen(s));

}

void Mp3::Action(string action,int frames)
{
char s[10];
string sgn;

if(frames >= remainframe) frames = currentframe + remainframe;
if(frames > 0) sgn = "+";
else
if(frames < 0) sgn = "-";

frames = abs(frames);

if(frames != 0)
	{
	sprintf(s,"%d\n",frames);
	action += " " + sgn;
	action += s;
	}

write(sender,action.c_str(),action.length());
}

void Mp3::Kill()
{
kill(mpg123,SIGTERM);
}

bool Mp3::GetInfo()
{
int rcpt;
fd_set fdsr;
struct timeval tv;
tv.tv_sec = tv.tv_usec = 0;

FD_ZERO(&fdsr);
FD_SET(receiver, &fdsr);


if(select(receiver+1, &fdsr, NULL, NULL, &tv) == 1)
	while(1)
		{
		rcpt = read(receiver, buf+buflen, 1);
		if(rcpt <= 0)return false;
     		if (buf[buflen] == '\0' || buf[buflen] == '\n' || buflen == buffsize)
         		{
			buf[buflen] = '\0';
			buflen = 0;
           		return true;
         		};
        	buflen += 1;
		}

}

void Mp3::HandleInfo(map<int,int>&FrameInfo)
{
char *c1, *c2, *c3, *c4, *c5, *c6;
int last_frameremain;
double totaltime;
c1 = strtok(buf, " \t");
if (c1 && !strcasecmp(c1, "@F"))
	{
	c2 = c1 ? strtok(NULL, " \t") : NULL;
	c3 = c2 ? strtok(NULL, " \t") : NULL;
	c4 = c3 ? strtok(NULL, " \t") : NULL;
	c5 = c4 ? strtok(NULL, " \t") : NULL;
	c6 = c5 ? strtok(NULL, " \t") : NULL;
	currentframe = atoi(c2);
	remainframe = atoi(c3);
	totaltime = atof(c4)+atof(c5);
	FrameInfo[0] = (int)totaltime;
	FrameInfo[1] = atoi(c4);
	}
else
if(c1 && !strcasecmp(c1, "@S"))
	{
	c2 = c1 ? strtok(NULL, " \t") : NULL;
	FrameInfo[2] = atoi(c2);
 	FrameInfo[3] = atoi(c2 = c2 ? strtok(NULL, " \t") : NULL);
	FrameInfo[4] = atoi(c2 = c2 ? strtok(NULL, " \t") : NULL);
	FrameInfo[5] = atoi(c2 = c2 ? strtok(NULL, " \t") : NULL);
	FrameInfo[6] = atoi(c2 = c2 ? strtok(NULL, " \t") : NULL);
	FrameInfo[7] = atoi(c2 = c2 ? strtok(NULL, " \t") : NULL);
	FrameInfo[8] = atoi(c2 = c2 ? strtok(NULL, " \t") : NULL);
	FrameInfo[9] = atoi(c2 = c2 ? strtok(NULL, " \t") : NULL);
	FrameInfo[10] = atoi(c2 = c2 ? strtok(NULL, " \t") : NULL);
	FrameInfo[11] = atoi(c2 = c2 ? strtok(NULL, " \t") : NULL);
	FrameInfo[12] = atoi(c2 = c2 ? strtok(NULL, " \t") : NULL);
	FrameInfo[13] =  atoi(c2 = c2 ? strtok(NULL, " \t") : NULL);
      }
else
if (c1 && !strcasecmp(c1, "@P"))
	{
	c2 = c1 ? strtok(NULL, " \t") : NULL;
	int status = atoi(c2);
	switch (status)
		{
		case 0 :
			if(lst >ready)
				{
				sleep(1);
				Open("");
				}
			break;
		case 1 :
			break;
		case 2 :
			break;
		}
	}

}

bool Mp3::Add(string File2Add)
{
Quee[lst++] = File2Add;
}


